library(Benchmarking)
##define data##
x <- matrix(c(100,200,300,500,100,200,600),ncol=1)
y <- matrix(c(75,100,300,400,25,50,400),ncol=1)

##DEA plots##
dea.plot(x,y, main="Basic plot of frontier")
dea.plot.frontier(x,y,txt=TRUE)
dea.plot(x,y,RTS="crs",ORIENTATION="in-out",txt=rownames(x))
dea.plot(x,y,RTS="vrs",ORIENTATION="in-out",txt=rownames(x))
dea.plot(x,y,RTS="fdh",ORIENTATION="in-out",txt=rownames(x),main="fdh")
dea.plot(x,y,RTS="irs",ORIENTATION="in-out",txt=TRUE,main="irs")

##3d plot## required first to have the package
require(scatterplot3d)
with(mf2011, scatterplot3d(x1,x2,y))
##DEA calculation-VRS-Input##
dea(x,y, RTS="vrs", ORIENTATION="in")
e <- dea(x,y)
eff(e)
print(e)
summary(e)
peers(e)
get.number.peers(e)

##DEA calculation-CRS-Input##
dea(x,y, RTS="crs", ORIENTATION="in")
e1 <- dea(x,y)
eff(e1)
print(e1)
summary(e1)
peers(e1)
get.number.peers(e1)

##DEA calculation-VRS-Output##
dea(x,y, RTS="vrs", ORIENTATION="out")
e2 <- dea(x,y)
eff(e2)
print(e2)
summary(e2)
peers(e2)
get.number.peers(e2)

##DEA calculation-CRS-output##
dea(x,y, RTS="crs", ORIENTATION="out")
e3 <- dea(x,y)
eff(e3)
print(e3)
summary(e3)
peers(e3)
get.number.peers(e3)


# Who are the firms that firm 1 and 4 is peers for
get.which.peers(e, c(11,41))
lambda(e)
##Input savings##
(1-eff(e)) * x

##Slacks##
e11<- dea(x,y,SLACK=TRUE)
?data.frame(e1$eff,el$eff,el$slack,el$sx,el$sy)?
data.frame(eff(e11),e11$slack,e11$sx,e11$sy,lambda(e11))

##Fully efficient units##
which(eff(e) == 1 & !el$slack)

##Density plot##
eff.dens.plot(e)
den <- eff.dens(e)
lines(den,lw=2)

##FDH##
dea(x,y,RTS="fdh+",param=c(.7,1.2))

##Super Efficiency##
esuper <- sdea(x,y, RTS="vrs", ORIENTATION="in")
esuper
print(peers(esuper,NAMES=TRUE),quote=FALSE)

### With Real Data####
data(mf2011)
x <- with(mf2011, cbind(x1,x2,x3))
y <- with(mf2011, cbind(y1))
dea.plot(x,y, main="Basic plot of frontier")
w<-dea(x,y, RTS="vrs", ORIENTATION="in")
print(w)
summary(w)

w2<-dea(x,y, RTS="crs", ORIENTATION="in")
print(w2)


w1<-dea(x,y, RTS="vrs", ORIENTATION="out")
print(w1)
summary(w1)
dea(x,y, RTS="crs", ORIENTATION="in")
##output orientation##
dea(x,y, RTS="crs", ORIENTATION="out")
dea(x,y, RTS="vrs", ORIENTATION="out")
dea1 <-dea(x,y, RTS="vrs", ORIENTATION="out")
eff(dea1)

##FDH##
fdh1 <-dea(x,y,RTS="fdh+",param=c(.7,1.2))
eff(fdh1)

##Super Efficiency##
esuper <- sdea(x,y, RTS="vrs", ORIENTATION="in")
esuper
print(peers(esuper,NAMES=TRUE),quote=FALSE)

##Outlier Detection####
tap <- outlier.ap(x,y, NDEL=2)
print(cbind(tap$imat,tap$rmin), na.print="", digit=2)
outlier.ap.plot(tap$ratio)


## Test of hypothesis in DEA model###
# Null hypothesis is that technology is CRS and the alternative is VRS###
ec <- dea(x,y, RTS="crs")
Ec <- eff(ec)
ev <- dea(x,y, RTS="vrs")
Ev <- eff(ev)

# The test statistic; equation (6.1)
S <- sum(Ec)/sum(Ev)
print(S)

###Bootstrap####
# To calculate CRS and VRS efficiencies in the same bootstrap replicas
# we reset the random number generator before each call of the function dea.boot.
# To get the an initial value for the random number generating process
# we save its state (seed)
save.seed <- sample.int(1e9,1)

# The bootstrap and calculate CRS and VRS under the assumption that
# the true technology is CRS (the null hypothesis) and such that the
# results correponds to the case where CRS and VRS are calculated for
# the same reference set of firms; to make this happen we set the
# random number generator to the same state before the calls.
set.seed(save.seed)
nper <-1
nper <-200
bc <- dea.boot(x,y, nrep,, RTS="crs")
set.seed(save.seed)
bv <- dea.boot(x,y, nrep,, RTS="vrs", XREF=x,YREF=y, EREF=ec$eff)

# Calculate the statistic for each bootstrap replica
bs <- colSums(bc$boot)/colSums(bv$boot)
print(bs)
# The critical value for the test (default size \code{alpha} of test is 5%)
critValue(bs, alpha=.1)

# Accept the hypothesis at 10% level?
critValue(bs, alpha=.1) <= S

# The probability of observering a smaller value of S when the
# hypothesis is true; the p--value.
typeIerror(S, bs)
# Accept the hypothesis at size level 10%?
typeIerror(S, bs) >= .10
# The critical value for two-sided test in normal distribution found by simulation.
x <- rnorm(1000000)
critValue(x,.975)

##DEA optimal cost, revenue, and profit###
x <- matrix(c(2,12, 2,8, 5,5, 10,4, 10,6, 3,13), ncol=2, byrow=TRUE)
y <- matrix(1,nrow=dim(x)[1],ncol=1)
w <- matrix(c(1.5, 1),ncol=2)
txt <- LETTERS[1:dim(x)[1]]
dea.plot(x[,1],x[,2], ORIENTATION="in",  cex=1.25)
text(x[,1],x[,2],txt,adj=c(-.7,-.2),cex=1.25)
## technical efficiency##
te <- dea(x,y,RTS="vrs")
xopt <- cost.opt(x,y,w,RTS=1)
cobs <- x %*% t(w)
copt <- xopt$x %*% t(w)
## cost efficiency##
ce <- copt/cobs
# allocaltive efficiency##
ae <- ce/te$eff
data.frame("ce"=ce,"te"=te$eff,"ae"=ae)
print(cbind("ce"=c(ce),"te"=te$eff,"ae"=c(ae)),digits=2)
## isocost line in the technology plot##
abline(a=copt[1]/w[2], b=-w[1]/w[2], lty="dashed")

##Calculates additive efficiency as sum of input and output slacks within different DEA models ##
sb <- dea.add(x,y,RTS="vrs")
data.frame("sx"=sb$sx,"sy"=sb$sy,"sum"=sb$sum,"slack"=sb$slack)

