## use R as a pocket calculator
3 + 4
3 - 4
3 * 4
3 / 4
3^4

## precedence rule and parenthesis
3 + 4 * 2
( 3 + 4 ) * 2
3 + 4 * 2^2
( 3 + 4 * 2 )^2

## mathematical functions
log( 10 )  # natural logarithm
exp( 1 )  # exponential function
sqrt( 49 )  # square root

## variables
x <- 3 + 6
x
x * 2
sqrt( x )

## vectors
x <- c( 1, 2, 3, 4 )
x
x[ 2 ]
x[ c( 2, 3 ) ]
x[ -2 ]
x[ -c( 2, 3 ) ]
1:10
x <- 1:10
x
length( x )

## simple statistical functions
min( x )
max( x )
range( x )
sum( x )
mean( x )
median( x )
var( x )
sd( x )
sqrt( var( x ) )

## (pseudo) random numbers
rnorm( 5 )  # from standard normal distribution
runif( 5 )  # from uniform distribution
set.seed( 134 )
rnorm( 5 )  # all should get the same pseudo random numbers

## linear regression
y <- 2 + 0.5 * x + rnorm( 10 )
lm( y ~ 1 + x )
lm( y ~ x )
lm( y ~ x - 1 )
lm( y ~ 0 + x )
ols <- lm( y ~ x )
summary( ols )
coef( ols )
coef( summary( ols ) )
residuals( ols )
fitted( ols )
fitted( ols ) + residuals( ols )
y

## plotting
# simple plot
plot( y )
# changing colour
plot( y, col = "blue" )
# changing plotting symbols (characters)
plot( y, pch = 19 )
plot( y, pch = 23 )
# changing size of plotting symbols
plot( y, cex = 3 )
plot( y, cex = 0.5 )
# changing type
plot( y, type = "l" )
plot( y, type = "o" )
plot( y, type = "s" )
# changing line type
plot( y, type = "l", lty = "dashed" )
plot( y, type = "l", lty = "dotted" )
# changing line width
plot( y, type = "l", lwd = 3 )
# changing labels
plot( y, xlab = "year", ylab = "yield" )
# changing plotting limits
plot( y, xlim = c( 0, 20 ), ylim = c( -10, 30 ) )
# scatter plots
plot( x, y )
plot( x, fitted( ols ) )
plot( x, residuals( ols ) )
# histograms
hist( y )
hist( rnorm( 10000 ) )
# plots created with multiple commands
plot( x, y )
points( x, fitted( ols ), col = "red" )
abline( ols, col = "blue" )

## help system
?sd
??median

## matrices
m <- matrix( 1:6, nrow = 2 )
m
m <- matrix( 1:6, nrow = 2, byrow = TRUE )
m
m[ 2, 2 ]
m[ 2, ]
m[ , 2 ]
m[ , -2 ]
m[ , c( 1, 3 ) ]
m[ 2, 2 ] <- 9
nrow( m )
ncol( m )
dim( m )
rownames( m ) <- c( "a", "b" )
m
colnames( m ) <- c( "A", "B", "C" )
m


## data.frames
d <- data.frame( x = x, y = y )
d
d$x
d$y
d$z <- runif( 10 )
d
ols2 <- lm( y ~ x + z, data = d )

cbind( d$x, d$y, d$z ) 
rbind( d$x, d$y, d$z )

## work space
ls()  # list items in the workspace
rm( x )  # remove an item from the workspace
ls()
# save workspace: File -> Save workspace
# load saved workspace: File -> Load workspace

## case sensitivity
M <- 6
M
m  # M and m are NOT the same

## saving and reading data files
# change working directory: File -> Change dir
write.table( d, "myData.tab" )  # write our data frame to a data file
rm( d )  # remove the data frame from the work space
d <- read.table( "myData.tab" )  # read the data frame from file back to the work space
# CSV-format (comma separated values)
write.csv( d, "myData.csv" )
read.csv( "myData.csv" )
# European "CSV"-format (semicolon separated values, comma used for decimal points)
write.csv2( d, "myData2.csv" )
read.csv2( "myData2.csv" )

## R script files
# open new script: File -> New script
# open existing script: File -> Open script
# save script: select script window, File -> Save (as)
# execute R command(s): right mouse button -> Run line or selection (or: Ctrl-R)
# select all: Edit -> Select all (or: CTRL-A)

## multi-line commands
3 *
 4
sqrt(
 3
 )

## add-on packages
# install packages (only once): Packages -> Install package(s) from CRAN
# load package (every time you (re)start R):
library( "foreign" )
help( package = "foreign" )

## importing data from MS-Excel
# save the data in MS-Excel as a file in tab-delimited, CSV, or data interchange
# format and read it in R with read.delim / read.delim2, read.csv / read.csv2, or
# read.DIF, respectively.
# OR: read the MS-Excel files directly with the xlsReadWrite package
# (see http://treetron.googlepages.com/)

