# load data set
library( "frontier" )
data( "riceProdPhil" )

###################  DEA  ###########################
library( "Benchmarking" )

?dea 

# matrix of input quantities
xMat <- cbind( riceProdPhil$AREA, riceProdPhil$LABOR, riceProdPhil$NPK )

# vector of output quantities
yVec <- riceProdPhil$PROD

# input-oriented DEA with VRS
deaVrsIn <- dea( xMat, yVec )
hist( eff( deaVrsIn ) )
peers( deaVrsIn )[ 1:6, ]
lambda( deaVrsIn )[ 1:6, ]

# input-oriented DEA with CRS
deaCrsIn <- dea( xMat, yVec, RTS = "crs" )
hist( eff( deaCrsIn ) )

# scale efficiencies
se <- eff( deaCrsIn ) / eff( deaVrsIn )
hist( se )

# input-oriented DEA with DRS
deaDrsIn <- dea( xMat, yVec, RTS = "drs" )
hist( eff( deaDrsIn ) )

# check if firms are too small or too large
sum( eff( deaVrsIn ) - eff( deaDrsIn ) > 1e-4 )  # too small

# output-oriented DEA with VRS
deaVrsOut <- dea( xMat, yVec, ORIENTATION = "out" )
hist( eff( deaVrsOut ) )

# output-oriented DEA with CRS
deaCrsOut <- dea( xMat, yVec, RTS = "crs", ORIENTATION = "out" )
hist( eff( deaCrsOut ) )
all.equal( eff( deaCrsIn ), 1 / eff( deaCrsOut ) )

# super-efficiencies: input-oriented DEA with CRS
sdeaVrsIn <- sdea( xMat, yVec )
hist( eff( sdeaVrsIn ) )
# 
# # DEA with graph hyperbolic efficiencies and VRS
# deaVrsGraph <- dea( xMat, yVec, ORIENTATION = "graph" )
# hist( eff( deaVrsGraph ) )
# plot( eff( deaVrsIn ), eff( deaVrsGraph ) )
# abline(0,1)

# slack in input-oriented DEA with VRS
deaVrsIn <- dea( xMat, yVec, SLACK = TRUE )
sum( deaVrsIn$slack )
deaVrsIn$sx[ 1:6, ]
deaVrsIn$sy[ 1:6, ]


# obtaining and decomposing productivity changes
xMat0 <- xMat[ riceProdPhil$YEARDUM == 1, ]
xMat1 <- xMat[ riceProdPhil$YEARDUM == 2, ]
yVec0 <- yVec[ riceProdPhil$YEARDUM == 1 ]
yVec1 <- yVec[ riceProdPhil$YEARDUM == 2 ]

c00 <- eff( dea( xMat0, yVec0, RTS = "crs" ) )
c01 <- eff( dea( xMat0, yVec0, XREF = xMat1, YREF = yVec1, RTS = "crs" ) )
c11 <- eff( dea( xMat1, yVec1, RTS = "crs" ) )
c10 <- eff( dea( xMat1, yVec1, XREF = xMat0, YREF = yVec0, RTS = "crs" ) )

# productivity changes (Malmquist)
dProd0 <- c10 / c00
hist( dProd0 )
dProd1 <- c11 / c01
plot( dProd0, dProd1 )
dProd <- sqrt( dProd0 * dProd1 )
hist( dProd )

# technological changes
dTech0 <- c00 / c01
dTech1 <- c10 / c11
plot( dTech0, dTech1 )
dTech <- sqrt( dTech0 * dTech1 )
hist( dTech )

# efficiency changes
dEff <- c11 / c00
hist( dEff )

# checking Malmquist decomposition
all.equal( dProd, dTech * dEff )

