### LOAD THE REQUIRED PACKAGES
require(frontier)
require(plm)
require(lpSolve)

### OPEN THE DATA
data<-read.csv("full_est_data.csv")


### MEAN-SCALE THE VARIABLES
# assume y-median scaling rule

data$Y_star<-data$Y/data$Y[which(data$Y==median(data$Y))]
data$K_star<-data$K/data$K[which(data$Y==median(data$Y))]
data$L_star<-data$L/data$L[which(data$Y==median(data$Y))]
data$E_star<-data$E/data$E[which(data$Y==median(data$Y))]
data$T<-rep(rev((max(data$Year)-data$Year[which(data$Y==median(data$Y))]-(0:16))),103)

### CREATE THE GROUPS (BY LDC)
data$LDC<-ifelse(data$GNI<4086,1,0)

### CREATE PANEL DATA OBJECT FOR STOCHASTIC FRONTIER ESTIMATION
est.data<-plm.data( data, c( "Region" , "Year" ) )

### GENERATE FUNCTIONS TO ESTIMATE

translog<- (log(Y_star)~ log(K_star)+log(L_star)+log(E_star) + I(log(K_star)*log(K_star))+I(log(L_star)*log(L_star))+I(log(E_star)*log(E_star)) +I(log(K_star)*log(L_star))+I(log(K_star)*log(E_star))+I(log(L_star)*log(E_star)) )
translog_z_POP<- (log(Y_star)~ log(K_star)+log(L_star)+log(E_star) + I(log(K_star)*log(K_star))+I(log(L_star)*log(L_star))+I(log(E_star)*log(E_star)) +I(log(K_star)*log(L_star))+I(log(K_star)*log(E_star))+I(log(L_star)*log(E_star))|+POP )

translog_z_POP_technology_neutral<- (log(Y_star)~ log(K_star)+log(L_star)+log(E_star) + I(log(K_star)*log(K_star))+I(log(L_star)*log(L_star))+I(log(E_star)*log(E_star)) +I(log(K_star)*log(L_star))+I(log(K_star)*log(E_star))+I(log(L_star)*log(E_star)) + T + I(T*T) + I(T*T*T)|POP )
translog_z_POP_technology_factor <- (log(Y_star)~ log(K_star)+log(L_star)+log(E_star) + I(log(K_star)*log(K_star))+I(log(L_star)*log(L_star))+I(log(E_star)*log(E_star)) +I(log(K_star)*log(L_star))+I(log(K_star)*log(E_star))+I(log(L_star)*log(E_star)) + T + I(T*T) + I(T*T*T) + I(T*log(K_star))+I(T*log(L_star))+I(T*log(E_star))|POP )

### ESTIMATE STOCHASTIC FRONTIER MODELS
summary(sfa(translog, data=est.data, timeEffect=TRUE))
summary(sfa(translog_z_POP, data=est.data, timeEffect=TRUE))

#   ESTIMATE SAME FUNCTIONS BY GROUP
M1_LDC<-summary(sfa(translog_z_POP, data=est.data[which(est.data$LDC==1),], timeEffect=TRUE))
M1_DC<-summary(sfa(translog_z_POP, data=est.data[which(est.data$LDC==0),], timeEffect=TRUE))


M1_LDC_tech<-summary(sfa(translog_z_POP_technology_factor, data=est.data[which(est.data$LDC==1),], timeEffect=TRUE))
M1_DC_tech<-summary(sfa(translog_z_POP_technology_factor, data=est.data[which(est.data$LDC==0),], timeEffect=TRUE))


plot(c(t(M1_LDC$resid)))

plot(c(t(M1_LDC_tech$resid)))
plot(c(t(M1_DC_tech$resid)))


#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#
###    BOOTSTRAP META-FRONTIER    ###
#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#

### GENERAL PARAMETERS FOR BOOTSTRAP
initial_coefs<-t(cbind(M1_LDC$mleParam,M1_DC$mleParam))[c(1,5),1:10]
rownames(initial_coefs)<-c("1","0")


meta_group_id<-unique(data$LDC)
x_vars<-c("K","L","E")
boot_reps<-5000
boot_coefs<-matrix(0,boot_reps,length(initial_coefs[1,]))

time<-system.time(
for (i in 1:boot_reps){

boot_data<-data
boot_data$K<-data$K_star
boot_data$L<-data$L_star
boot_data$E<-data$E_star

### GENERATE THE BOOTSTRAP SAMPLES
for (k in meta_group_id){
for (j in x_vars){
boot_data[which(boot_data$LDC==k), gsub(" ","",paste(j,"_",i))]<-
sample(boot_data[which(boot_data$LDC==k),j])
}}

##  NOW LOG THE VARIABLES AND CREATE THE SQUARES AND CROSS PRODUCTS NEEDED FOR ESTIMATION
for (j in x_vars){
# LOG THE VARIABLES
boot_data[, gsub(" ","",paste(j,"_",i))]<- log(boot_data[, gsub(" ","",paste(j,"_",i))])

# CREATE SQUARED TERMS
boot_data[, gsub(" ","",paste(j,j,"_",i))]<- (boot_data[, gsub(" ","",paste(j,"_",i))])^2
}

# CREATE CROSS PRODUCTS
for (j in x_vars){
for(z in  x_vars[which(x_vars!=j)]){ 
boot_data[, gsub(" ","",paste(j,z,"_",i))]<-(boot_data[, gsub(" ","",paste(j,"_",i))]*boot_data[, gsub(" ","",paste(z,"_",i))])
}}

## CALCULATE FITTED FRONTIER VALUES FOR EACH GROUP/REPLICATION
for (k in meta_group_id){

boot_data[which(boot_data$LDC==k),gsub(" ","",paste("efficient_meta",i))]<-(
initial_coefs[paste(k),1]+
(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("K_",i))] *initial_coefs[paste(k),2])+
(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("L_",i))] *initial_coefs[paste(k),3])+
(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("E_",i))] *initial_coefs[paste(k),4])+
(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("KK_",i))]*initial_coefs[paste(k),5])+
(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("LL_",i))]*initial_coefs[paste(k),6])+
(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("EE_",i))]*initial_coefs[paste(k),7])+
(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("KL_",i))]*initial_coefs[paste(k),8])+
(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("KE_",i))]*initial_coefs[paste(k),9])+
(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("LE_",i))]*initial_coefs[paste(k),10]))

}

### EVALUATE THE LINEAR PROGRAM FOR EACH BOOTSTRAP REPLICATION
f.dir <- rep(">=", length(boot_data[,"Y"])) 
f.obj<-c(1,
mean(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("K_",i))]),
mean(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("L_",i))]),
mean(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("E_",i))]),
mean(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("KK_",i))]),
mean(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("LL_",i))]),
mean(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("EE_",i))]),
mean(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("KL_",i))]),
mean(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("KE_",i))]),
mean(boot_data[which(boot_data$LDC==k),gsub(" ","",paste("LE_",i))]))

f.con<-cbind(rep(1,length(boot_data[,"Y"])),
boot_data[,gsub(" ","",paste("K_",i))],
boot_data[,gsub(" ","",paste("L_",i))],
boot_data[,gsub(" ","",paste("E_",i))],
boot_data[,gsub(" ","",paste("KK_",i))],
boot_data[,gsub(" ","",paste("LL_",i))],
boot_data[,gsub(" ","",paste("EE_",i))],
boot_data[,gsub(" ","",paste("KL_",i))],
boot_data[,gsub(" ","",paste("KE_",i))],
boot_data[,gsub(" ","",paste("LE_",i))])

f.rhs <- boot_data[,gsub(" ","",paste("efficient_meta",i))]

boot_coefs[i,]<-matrix(lp ("min", f.obj, f.con, f.dir, f.rhs)$solution)

})

time

layout( rbind(cbind(1,2,3,4),cbind(5,6,7,8),cbind(9,10,11,12)) )
for(i in 1:10){hist(boot_coefs[,i], col="gray", main=colnames(initial_coefs)[i])}



### ANALYSE THE TIME TAKEN PER REPLICATION
(samples_time+transformations_time+linear_prog_time)/boot_reps



